package hu.kozadat.search.base.preprocessor;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * User: codex
 */
@Component("centuryRangePreprocessor")
public class CenturyRangePreprocessor extends QueryPreprocessor {

    private static final Pattern centuryRangePattern = Pattern.compile("(^|[\\s()])((isz|ie)[.]?[\\s]*)?([0-9]{1,2}|[ivx]{1,4})[.]?(-|/)([0-9]{1,2}|[ivx]{1,4})[.]?[\\s]*(század|sz[.]?)(\\b|\\B)", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);
    private static final Map<String, String> centuryConversionMap;

    static {
        centuryConversionMap = new HashMap<String, String>();
        centuryConversionMap.put("i", "1");
        centuryConversionMap.put("ii", "2");
        centuryConversionMap.put("iii", "3");
        centuryConversionMap.put("iv", "4");
        centuryConversionMap.put("v", "5");
        centuryConversionMap.put("vi", "6");
        centuryConversionMap.put("vii", "7");
        centuryConversionMap.put("viii", "8");
        centuryConversionMap.put("ix", "9");
        centuryConversionMap.put("x", "10");
        centuryConversionMap.put("xi", "11");
        centuryConversionMap.put("xii", "12");
        centuryConversionMap.put("xiii", "13");
        centuryConversionMap.put("xiv", "14");
        centuryConversionMap.put("xv", "15");
        centuryConversionMap.put("xvi", "16");
        centuryConversionMap.put("xvii", "17");
        centuryConversionMap.put("xviii", "18");
        centuryConversionMap.put("xix", "19");
        centuryConversionMap.put("xx", "20");
        centuryConversionMap.put("xxi", "21");
    }

    private DatePrecision datePrecision;

    public DatePrecision getDatePrecision() {
        return datePrecision;
    }

    @Autowired
    public void setDatePrecision(DatePrecision datePrecision) {
        this.datePrecision = datePrecision;
    }

    private String formatStartCentury(int century) {
        GregorianCalendar cal = new GregorianCalendar((century - 1) * 100 + 1, 0, 1);
        Date event = cal.getTime();

        if (datePrecision != null)
            event = datePrecision.adjustDateDownwards(event);

        return df.format(event);
    }

    private String formatEndCentury(int century) {
        GregorianCalendar cal = new GregorianCalendar(century * 100, 11, 31);
        Date event = cal.getTime();

        if (datePrecision != null)
            event = datePrecision.adjustDateUpwards(event);

        return df.format(event);
    }

    public QueryPreprocessorResult preprocess(String segment, boolean appendFieldName) {
        QueryPreprocessorResult res = new QueryPreprocessorResult();
        res.setOriginalQuery(segment);
        Matcher centuryRangeMatcher = centuryRangePattern.matcher(segment);
        StringBuffer sb = new StringBuffer();

        String fieldStr = appendFieldName ? "date:" : "";

        while (centuryRangeMatcher.find()) {
            //-- not null
            String startStr = centuryRangeMatcher.group(1);
            //-- can be null
            String timelinePartSelector = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(3)).toLowerCase();
            //-- not null
            String startCenturyStr = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(4)).toLowerCase();
            //-- not null
            String endCenturyStr = com.actiwise.einfoszab.Utils.null2String(centuryRangeMatcher.group(6)).toLowerCase();

            boolean isIsz = true;
            if (!"".equals(timelinePartSelector)) {
                if (BEFORE_CHRISTMAS.equals(timelinePartSelector))
                    isIsz = false;
            }

            if (centuryConversionMap.get(startCenturyStr) != null)
                startCenturyStr = centuryConversionMap.get(startCenturyStr);
            if (centuryConversionMap.get(endCenturyStr) != null)
                endCenturyStr = centuryConversionMap.get(endCenturyStr);

            int startCentury = 0;
            int endCentury = 0;
            boolean validRange = true;

            try {
                startCentury = Integer.parseInt(startCenturyStr);
                endCentury = Integer.parseInt(endCenturyStr);

                if (startCentury < 0 || endCentury < 0)
                    validRange = false;

                if (isIsz) {
                    if (startCentury > endCentury)
                        validRange = false;
                } else {
                    if (startCentury < endCentury)
                        validRange = false;
                }
            } catch (NumberFormatException nex) {
                validRange = false;
            }

            String modifiedCenturyRange;
            if (validRange)
                modifiedCenturyRange = startStr + fieldStr + "[" + formatStartCentury(startCentury) + " TO " + formatEndCentury(endCentury) + "]";
            else
                modifiedCenturyRange = startStr;

            centuryRangeMatcher.appendReplacement(sb, modifiedCenturyRange);
            res.setQueryModified(true);
        }
        centuryRangeMatcher.appendTail(sb);

        res.setFilteredQuery(sb.toString());
        return res;
    }
}
