package hu.kozadat.search.presentation.institutionsearch;

import hu.kozadat.search.Utils;
import hu.kozadat.search.model.InstitutionSearchResult;
import hu.kozadat.search.model.InstitutionSearchResultItem;
import hu.kozadat.search.persistence.DataProviderOperations;
import hu.kozadat.search.persistence.institutionsearch.InstitutionSearch;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.service.log.SearchLogger;
import hu.scriptum.einfoszab.datamodel.Region;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.support.PagedListHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * Intezmenyi kereses form kontroller-e.
 *
 * @author Csaba Horvath, Scriptum Inc.
 * @version $Id$
 */
@Controller
public class InstitutionSearchController {

    private static final String SESSION_ATTRIBUTE_INSTITUTION_SEARCH_RESULT = "institutionSearchResult";
    private static final String SESSION_ATTRIBUTE_PAGED_INSTITUTION_SEARCH_RESULT = "pagedInstitutionSearchResult";
    private InstitutionSearch institutionSearch;
    private SearchLogger searchLogger;
    private SearchUIConfiguration searchUIConfiguration;
    private DataProviderOperations dataProviderOperations;

    public static void clearSessionAttributes(HttpSession session) {
        if (session != null) {
            session.removeAttribute(SESSION_ATTRIBUTE_INSTITUTION_SEARCH_RESULT);
            session.removeAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_SEARCH_RESULT);
        }
    }

    @ModelAttribute("institutionSearchFormBackingBean")
    protected InstitutionSearchFormBackingBean createInstitutionSearchFormBackingBean() {
        InstitutionSearchFormBackingBean institutionSearchFormBackingBean = new InstitutionSearchFormBackingBean();

        List<LetterGroup> availableLetterGroups = new LinkedList<LetterGroup>();
        availableLetterGroups.add(LetterGroup.A_F);
        availableLetterGroups.add(LetterGroup.G_L);
        availableLetterGroups.add(LetterGroup.M_R);
        availableLetterGroups.add(LetterGroup.S_Z);
        availableLetterGroups.add(LetterGroup.SPECIAL);

        Map<String, Long> regionIdMap = new TreeMap<String, Long>();
        List<Region> regions = dataProviderOperations.getRegions();
        for (Region region : regions) {
            regionIdMap.put(Utils.removeAccents(region.getName().toLowerCase()), region.getId());
        }
        institutionSearchFormBackingBean.setRegionIdMap(regionIdMap);

        institutionSearchFormBackingBean.setAvailableLetterGroups(availableLetterGroups);
        return institutionSearchFormBackingBean;
    }

    @RequestMapping(value = "/intezmeny")
    public ModelAndView showForm(HttpServletRequest httpServletRequest) throws Exception {

        HttpSession session = httpServletRequest.getSession();

        clearSessionAttributes(session);

        InstitutionSearchFormBackingBean form = createInstitutionSearchFormBackingBean();

        String viewName = "institutionSearch";
        ModelAndView modelAndView = new ModelAndView(viewName);
        modelAndView.addObject("institutionSearchFormBackingBean", form);
        return modelAndView;
    }

    @RequestMapping(value = "/intezmeny/talalatok")
    public ModelAndView searchAndShowResult(
            HttpServletRequest httpServletRequest,
            @RequestParam(value = "oldal", required = false) Integer page,
            @ModelAttribute("institutionSearchFormBackingBean") InstitutionSearchFormBackingBean form
    ) throws Exception {

        int pageSize = searchUIConfiguration.getInstitutionSearchResultPageSize();

        LetterGroup letters = LetterGroup.parse(form.getCsoport());

        String viewName;
        InstitutionSearchResult institutionSearchResult = (InstitutionSearchResult) httpServletRequest.getSession().getAttribute(SESSION_ATTRIBUTE_INSTITUTION_SEARCH_RESULT);
        PagedListHolder<InstitutionSearchResultItem> pagedInstitutionSearchResult = (PagedListHolder<InstitutionSearchResultItem>) httpServletRequest.getSession().getAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_SEARCH_RESULT);

        if (institutionSearchResult == null || pagedInstitutionSearchResult == null || institutionSearchResult.hasFormChange(form)) {
            if (letters != null) {

                institutionSearchResult = institutionSearch.searchByLeadingLetters(letters.getLettersArray());
                institutionSearchResult.setFormHashCode(form);
                institutionSearchResult.setByLetters(true);

                searchLogger.addInstitutionSearchLog(httpServletRequest.getRemoteAddr(), letters.getLabel(), null, 0, institutionSearchResult.getItems().size());

            } else if (form.getNev() != null && !"".equals(form.getNev().trim())) {

                institutionSearchResult = institutionSearch.search(form.getNev());
                institutionSearchResult.setFormHashCode(form);
                institutionSearchResult.setByName(true);

                searchLogger.addInstitutionSearchLog(httpServletRequest.getRemoteAddr(), form.getNev(), null, 0, institutionSearchResult.getItems().size());

            } else {
                return new ModelAndView("redirect:/intezmeny");
            }

            pagedInstitutionSearchResult = new PagedListHolder<InstitutionSearchResultItem>();
            pagedInstitutionSearchResult.setPageSize(pageSize);
            pagedInstitutionSearchResult.setSource(institutionSearchResult.getItems());

            httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_INSTITUTION_SEARCH_RESULT, institutionSearchResult);
            httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_SEARCH_RESULT, pagedInstitutionSearchResult);

        }

        if (page == null || page < 1) {
            page = 1;
        }
        if (page > pagedInstitutionSearchResult.getPageCount()) {
            page = pagedInstitutionSearchResult.getPageCount();
        }
        pagedInstitutionSearchResult.setPage(page - 1);

        if (institutionSearchResult.isByLetters()) {
            viewName = "institutionSearchResultByLetterGroup";
        } else if (institutionSearchResult.isByName()) {
            viewName = "institutionSearchResultByName";
        } else {
            return new ModelAndView("redirect:/intezmeny");
        }

        ModelAndView modelAndView = new ModelAndView(viewName);

        modelAndView.addObject(SESSION_ATTRIBUTE_INSTITUTION_SEARCH_RESULT, institutionSearchResult);
        modelAndView.addObject(SESSION_ATTRIBUTE_PAGED_INSTITUTION_SEARCH_RESULT, pagedInstitutionSearchResult);

        return modelAndView;
    }

    @RequestMapping("/intezmeny/telepules/{dataProviderAlias}")
    public ModelAndView showDataProvidersBySettlement(@PathVariable("dataProviderAlias") String dataProviderAlias) {
        String viewName = "";
        ModelAndView modelAndView = new ModelAndView(viewName);
        return modelAndView;
    }

    @RequestMapping("/intezmeny/kisterseg/{subRegionAlias}")
    public ModelAndView showDataProvidersBySubRegion(@PathVariable("subRegionAlias") String subRegionAlias) {
        String viewName = "";
        ModelAndView modelAndView = new ModelAndView(viewName);
        return modelAndView;
    }

    @RequestMapping("/intezmeny/megye/{countyAlias}")
    public ModelAndView showDataProvidersByCounty(@PathVariable("countyAlias") String countyAlias) {
        String viewName = "";
        ModelAndView modelAndView = new ModelAndView(viewName);
        return modelAndView;
    }

    @RequestMapping("/intezmeny/regio/{regionAlias}")
    public ModelAndView showDataProvidersByRegion(@PathVariable("regionAlias") String regionAlias) {
        String viewName = "";
        ModelAndView modelAndView = new ModelAndView(viewName);
        return modelAndView;
    }

    @Autowired
    @Qualifier("institutionSearch")
    public void setInstitutionSearch(InstitutionSearch institutionSearch) {
        this.institutionSearch = institutionSearch;
    }

    @Autowired
    @Qualifier("searchLogger")
    public void setSearchLogger(SearchLogger searchLogger) {
        this.searchLogger = searchLogger;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }

    @Autowired
    public void setDataProviderOperations(DataProviderOperations dataProviderOperations) {
        this.dataProviderOperations = dataProviderOperations;
    }
}
