package hu.kozadat.search.presentation.interactivesearch;

import hu.kozadat.search.interactive.InteractiveSearchExpression;
import hu.kozadat.search.interactive.InteractiveSearchTermUtils;
import hu.kozadat.search.model.IndexerSearchResult;
import hu.kozadat.search.model.IndexerSearchResultItem;
import hu.kozadat.search.persistence.DataProviderOperations;
import hu.kozadat.search.presentation.CacheAwareController;
import hu.kozadat.search.presentation.UrlUtils;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.service.interactivesearch.InteractiveSearch;
import hu.kozadat.search.service.log.SearchLogger;
import hu.scriptum.einfoszab.datamodel.DataProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.support.PagedListHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Map;

@Controller
@SessionAttributes({"interactiveSearchFormBackingBean"})
public class InteractiveSearchController implements CacheAwareController {

    private static final Logger LOGGER = LoggerFactory.getLogger(InteractiveSearchController.class);

    private static final String SESSION_ATTRIBUTE_INTERACTIVE_SEARCH_RESULT = "interactiveSearchResult";
    private static final String SESSION_ATTRIBUTE_PAGED_INTERACTIVE_SEARCH_RESULT = "pagedInteractiveSearchResult";
    private static final String SESSION_ATTRIBUTE_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT = "institutionRecordInteractiveSearchResult";
    private static final String SESSION_ATTRIBUTE_PAGED_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT = "pagedInstitutionRecordInteractiveSearchResult";

    private DataProviderOperations dataProviderOperations;
    private SearchLogger searchLogger;
    private InteractiveSearch interactiveSearch;
    private SearchUIConfiguration searchUIConfiguration;

    private Map cachedReferenceData;

    public static void clearSessionAttributes(HttpSession session) {
        if (session != null) {
            session.removeAttribute(SESSION_ATTRIBUTE_INTERACTIVE_SEARCH_RESULT);
            session.removeAttribute(SESSION_ATTRIBUTE_PAGED_INTERACTIVE_SEARCH_RESULT);
            session.removeAttribute(SESSION_ATTRIBUTE_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT);
            session.removeAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT);
        }
    }

    @ModelAttribute("interactiveSearchFormBackingBean")
    protected InteractiveSearchFormBackingBean createInteractiveSearchFormBackingBean() {
        InteractiveSearchFormBackingBean form = new InteractiveSearchFormBackingBean();
        return form;
    }

    @RequestMapping("/osszetett-kereso")
    public ModelAndView showForm(HttpServletRequest httpServletRequest, @RequestParam(value = "modositas", required = false) String modifyParam, @ModelAttribute("interactiveSearchFormBackingBean") InteractiveSearchFormBackingBean form) {

        boolean modifyCurrentTerms = modifyParam != null;

        if (!modifyCurrentTerms) {

            form.reset();

            clearSessionAttributes(httpServletRequest.getSession());

        }

        String viewName = "interactiveSearch";

        ModelAndView modelAndView = new ModelAndView(viewName);

        return modelAndView;
    }

    @RequestMapping("/osszetett-kereso/talalatok")
    public ModelAndView searchAndShowResult(HttpServletRequest httpServletRequest, @RequestParam(value = "oldal", required = false) Integer page, @RequestParam(value = PARAM_NAME_CLEAR_CACHE, required = false) String clearCache, @ModelAttribute("interactiveSearchFormBackingBean") InteractiveSearchFormBackingBean form) throws Exception {

        LOGGER.debug("InteractiveSearchController.searchAndShowResult(): searchTerms=" + form.getSearchTerms());

        HttpSession session = httpServletRequest.getSession();
        session.setMaxInactiveInterval(1800);

        if (PARAM_VALUE_CLEAR_CACHE.equals(clearCache)) {
            clearSessionAttributes(session);
        }

        String sessionId = httpServletRequest.getRemoteHost() + "_" + httpServletRequest.getSession().getId();

        form.setDataProviderName(null);

        IndexerSearchResult indexerSearchResult;
        PagedListHolder<IndexerSearchResultItem> pagedIndexerSearchResult;

        indexerSearchResult = (IndexerSearchResult) session.getAttribute(SESSION_ATTRIBUTE_INTERACTIVE_SEARCH_RESULT);
        pagedIndexerSearchResult = (PagedListHolder<IndexerSearchResultItem>) session.getAttribute(SESSION_ATTRIBUTE_PAGED_INTERACTIVE_SEARCH_RESULT);

        boolean formChange = false;
        if (indexerSearchResult != null) {
            formChange = indexerSearchResult.hasFormChange(form);
        }

        if (indexerSearchResult == null || pagedIndexerSearchResult == null || formChange) {

            if (form.getSearchTerms() == null || "".equals(form.getSearchTerms().trim())) {
                return new ModelAndView("redirect:/osszetett-kereso");
            }

            InteractiveSearchExpression searchExpression = InteractiveSearchTermUtils.fromJSONString(form.getSearchTerms());

            indexerSearchResult = interactiveSearch.search(searchExpression, sessionId);
            indexerSearchResult.setFormHashCode(form);

            searchLogger.addInteractiveSearchLog(searchExpression);

            interactiveSearch.close(sessionId);

            pagedIndexerSearchResult = new PagedListHolder<IndexerSearchResultItem>();
            pagedIndexerSearchResult.setPageSize(searchUIConfiguration.getSearchResultPageSize());
            pagedIndexerSearchResult.setSource(indexerSearchResult.getFirstItems());

            httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_INTERACTIVE_SEARCH_RESULT, indexerSearchResult);
            httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_PAGED_INTERACTIVE_SEARCH_RESULT, pagedIndexerSearchResult);

        }

        if (page == null || page < 1) {
            page = 1;
        }
        if (page > pagedIndexerSearchResult.getPageCount()) {
            page = pagedIndexerSearchResult.getPageCount();
        }
        pagedIndexerSearchResult.setPage(page - 1);

        String viewName = "interactiveSearchResult";
        ModelAndView modelAndView = new ModelAndView(viewName);

        modelAndView.addObject("searchResult", indexerSearchResult);
        modelAndView.addObject("pagedSearchResult", pagedIndexerSearchResult);

        return modelAndView;
    }

    @RequestMapping("/osszetett-kereso/talalatok/{dataProviderId}")
    public ModelAndView searchAndShowResultByDataProvider(HttpServletRequest httpServletRequest, @PathVariable("dataProviderId") String dataProviderIdStr, @RequestParam(value = "oldal", required = false) Integer page, @RequestParam(value = PARAM_NAME_CLEAR_CACHE, required = false) String clearCache, @ModelAttribute("interactiveSearchFormBackingBean") InteractiveSearchFormBackingBean form) throws Exception {

        LOGGER.debug("InteractiveSearchController.searchAndShowResult(): searchTerms=" + form.getSearchTerms());

        HttpSession session = httpServletRequest.getSession();
        session.setMaxInactiveInterval(1800);

        if (PARAM_VALUE_CLEAR_CACHE.equals(clearCache)) {
            clearSessionAttributes(session);
            String viewName = UrlUtils.redirectTo(UrlUtils.excludeParam("/osszetett-kereso/talalatok/" + dataProviderIdStr, httpServletRequest, PARAM_NAME_CLEAR_CACHE));
            return new ModelAndView(viewName);
        }

        String sessionId = httpServletRequest.getRemoteHost() + "_" + httpServletRequest.getSession().getId();

        Long dataProviderId = null;
        try {
            dataProviderId = Long.parseLong(dataProviderIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("InstitutionRecordsController.showResultByDataProvider(): cannot parse '" + dataProviderIdStr + "', " + e.getMessage());
        }

        if (dataProviderId == null) {
            return new ModelAndView("redirect:/");
        }

        form.setDataProviderId(dataProviderId);

        IndexerSearchResult institutionRecordInteractiveSearchResult;
        IndexerSearchResult interactiveSearchResult;
        PagedListHolder<IndexerSearchResultItem> pagedInstitutionRecordInteractiveSearchResult;

        pagedInstitutionRecordInteractiveSearchResult = (PagedListHolder<IndexerSearchResultItem>) session.getAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT);
        institutionRecordInteractiveSearchResult = (IndexerSearchResult) session.getAttribute(SESSION_ATTRIBUTE_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT);
        interactiveSearchResult = (IndexerSearchResult) session.getAttribute(SESSION_ATTRIBUTE_INTERACTIVE_SEARCH_RESULT);

        boolean doSearch;
        if (institutionRecordInteractiveSearchResult == null || pagedInstitutionRecordInteractiveSearchResult == null) {
            if (interactiveSearchResult != null && interactiveSearchResult.getItemsByDataProviderIds().containsKey(dataProviderId)) {
                institutionRecordInteractiveSearchResult = interactiveSearchResult.getItemsByDataProviderAsSearchResult(dataProviderId);
                institutionRecordInteractiveSearchResult.setFormHashCode(interactiveSearchResult.getFormHashCode());

                httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT, institutionRecordInteractiveSearchResult);

                pagedInstitutionRecordInteractiveSearchResult = new PagedListHolder<IndexerSearchResultItem>();
                pagedInstitutionRecordInteractiveSearchResult.setPageSize(searchUIConfiguration.getSearchResultPageSize());
                if (institutionRecordInteractiveSearchResult.getItemsByDataProviderIds().containsKey(dataProviderId)) {
                    Collection<IndexerSearchResultItem> collection = institutionRecordInteractiveSearchResult.getItemsByDataProviderIds().get(dataProviderId);
                    pagedInstitutionRecordInteractiveSearchResult.setSource(new LinkedList<>(collection));
                }

                httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT, pagedInstitutionRecordInteractiveSearchResult);

            }
        }
        doSearch = institutionRecordInteractiveSearchResult == null || institutionRecordInteractiveSearchResult.hasFormChange(form);

        // tnagy search if first request or conditions changed
        if (doSearch) {

            if (form.getSearchTerms() == null || "".equals(form.getSearchTerms().trim())) {
                return new ModelAndView("redirect:/osszetett-kereso");
            }

            InteractiveSearchExpression searchExpression = InteractiveSearchTermUtils.fromJSONString(form.getSearchTerms());

            institutionRecordInteractiveSearchResult = interactiveSearch.search(searchExpression, sessionId);
            institutionRecordInteractiveSearchResult.setFormHashCode(form);

            searchLogger.addInteractiveSearchLog(searchExpression);

            interactiveSearch.close(sessionId);

            httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT, institutionRecordInteractiveSearchResult);


            pagedInstitutionRecordInteractiveSearchResult = new PagedListHolder<IndexerSearchResultItem>();
            pagedInstitutionRecordInteractiveSearchResult.setPageSize(searchUIConfiguration.getSearchResultPageSize());
            if (institutionRecordInteractiveSearchResult.getItemsByDataProviderIds().containsKey(dataProviderId)) {
                Collection<IndexerSearchResultItem> collection = institutionRecordInteractiveSearchResult.getItemsByDataProviderIds().get(dataProviderId);
                pagedInstitutionRecordInteractiveSearchResult.setSource(new LinkedList<>(collection));
            }

            httpServletRequest.getSession().setAttribute(SESSION_ATTRIBUTE_PAGED_INSTITUTION_RECORD_INTERACTIVE_SEARCH_RESULT, pagedInstitutionRecordInteractiveSearchResult);
        }

        // tnagy get dataprovider name from result or from dataprovider db table if result was empty
        IndexerSearchResultItem firstItem = null;
        if (institutionRecordInteractiveSearchResult != null && institutionRecordInteractiveSearchResult.getItemsByDataProviderIds().get(dataProviderId) != null) {
            Collection<IndexerSearchResultItem> collection = institutionRecordInteractiveSearchResult.getItemsByDataProviderIds().get(dataProviderId);
            firstItem = new LinkedList<>(collection).get(0);
            if (firstItem != null) {
                form.setDataProviderName(firstItem.getDataProviderName());
            }
        }
        if (firstItem == null) {
            DataProvider dataProvider = dataProviderOperations.getDataProvider(dataProviderId);
            if (dataProvider == null) {
                return new ModelAndView("redirect:/");
            }
            form.setDataProviderName(dataProvider.getInstitution().getName());
        }

        if (page == null || page < 1) {
            page = 1;
        }
        if (page > pagedInstitutionRecordInteractiveSearchResult.getPageCount()) {
            page = pagedInstitutionRecordInteractiveSearchResult.getPageCount();
        }
        pagedInstitutionRecordInteractiveSearchResult.setPage(page - 1);

        String viewName = "institutionRecordInteractiveSearchResult";
        ModelAndView modelAndView = new ModelAndView(viewName);

        modelAndView.addObject("searchResult", institutionRecordInteractiveSearchResult);
        modelAndView.addObject("pagedInstitutionRecordSearchResult", pagedInstitutionRecordInteractiveSearchResult);
        modelAndView.addObject("dataProviderId", dataProviderId);

        return modelAndView;
    }

    @Autowired
    public void setDataProviderOperations(DataProviderOperations dataProviderOperations) {
        this.dataProviderOperations = dataProviderOperations;
    }

    @Autowired
    public void setSearchLogger(SearchLogger searchLogger) {
        this.searchLogger = searchLogger;
    }

    @Autowired
    public void setInteractiveSearch(InteractiveSearch interactiveSearch) {
        this.interactiveSearch = interactiveSearch;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }
}
