package hu.kozadat.search.service;

import hu.kozadat.search.model.IndexerSearchResultItem;
import hu.kozadat.search.model.IndexerSearchResultSchemaType;
import hu.kozadat.search.persistence.SearchCache;
import hu.kozadat.search.presentation.viewer.AttachmentViewerController;
import org.apache.commons.lang.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.06.18.
 * Time: 10:10
 * To change this template use File | Settings | File Templates.
 */
public class IndexerSearchResultParser {

    private static final Logger LOGGER = LoggerFactory.getLogger(IndexerSearchResultParser.class);

    private static final String EQUAL_SIGN = "=";
    private static final String COLON_SIGN = ":";
    private static final String SLASH_SIGN = "/";

    private static final String INDEXER_FIELD_PREFIX_SIGN = "$";

    private static final String INDEXER_FIELD_RECORD_ID = "$record_id";
    private static final String INDEXER_FIELD_DATA_PROVIDER_NAME = "$provider_name";
    private static final String INDEXER_FIELD_DATA_PROVIDER_ID = "$provider_id";
    private static final String INDEXER_FIELD_SCHEMA_NAME = "$schema_name";
    private static final String INDEXER_FIELD_MODIFICATION_DATE = "$modified";
    private static final String INDEXER_FIELD_MODIFICATION_DATE_PATTERN = "yyyyMMddHHmmss";

    private static final String PATTERN_SPAN_CLASS_HIGHLIGHT_HTTP_SPAN = "<span class=\"highlight\">http</span>://";
    private static final String PATTERN_SPAN_CLASS_HIGHLIGHT = "<span class=\"highlight";

    private SimpleDateFormat modificationDateParser = new SimpleDateFormat(INDEXER_FIELD_MODIFICATION_DATE_PATTERN);

    protected void parseIndexerField(String line, IndexerSearchResultItem item) {
        int equalSignIndex = line.indexOf(EQUAL_SIGN);

        String fieldName = line.substring(0, equalSignIndex);

        String fieldValue = StringEscapeUtils.unescapeHtml(line.substring(equalSignIndex + 1).trim());

        if (INDEXER_FIELD_RECORD_ID.equals(fieldName)) {
            item.setRecordId(fieldValue);
        } else if (INDEXER_FIELD_DATA_PROVIDER_NAME.equals(fieldName)) {
            item.setDataProviderName(fieldValue);
        } else if (INDEXER_FIELD_DATA_PROVIDER_ID.equals(fieldName)) {
            item.setDataProviderId(Long.parseLong(fieldValue));
        } else if (INDEXER_FIELD_SCHEMA_NAME.equals(fieldName)) {
            item.setSchemaName(fieldValue);
            String schemaName = SearchCache.stripHtmlTags(fieldValue);
            item.setSchemaType(IndexerSearchResultSchemaType.getSchemaByName(schemaName));
        } else if (INDEXER_FIELD_MODIFICATION_DATE.equals(fieldName)) {
            Date modificationDate = null;
            try {
                modificationDate = modificationDateParser.parse(fieldValue);
            } catch (ParseException e) {
                LOGGER.warn(e.getMessage(), e);
            }
            item.setModificationDate(modificationDate);
        }
    }

    protected void parseSchemaElement(String line, IndexerSearchResultItem item) {
        int colonIndex = line.indexOf(COLON_SIGN);

        String schemaElementName = line.substring(0, colonIndex);
        String schemaElementQualifier = "";
        if (schemaElementName.indexOf(SLASH_SIGN) > 0) {
            int slashIndex = schemaElementName.indexOf(SLASH_SIGN);
            schemaElementQualifier = schemaElementName.substring(slashIndex + 1).trim();
            schemaElementName = schemaElementName.substring(0, slashIndex).trim();
        }
        String schemaElementValue = line.substring(colonIndex + 1).trim();
        schemaElementValue = StringEscapeUtils.unescapeHtml(schemaElementValue.replaceAll(PATTERN_SPAN_CLASS_HIGHLIGHT_HTTP_SPAN, "http://"));

        item.addSchemaElement(schemaElementName, schemaElementQualifier, schemaElementValue);

    }

    protected void parseLine(String line, IndexerSearchResultItem item) {

        if (line.startsWith(INDEXER_FIELD_PREFIX_SIGN)) {
            parseIndexerField(line, item);
        } else {
            parseSchemaElement(line, item);
        }
    }        
    
    
    public IndexerSearchResultItem parse(String resultText) {

        IndexerSearchResultItem item = new IndexerSearchResultItem();
        
        String[] lines = resultText.split("\n");

        for (String line : lines) {

            line = line.trim();

            parseLine(line, item);

        }

        setAttachmentViewerInfo(item);

        return item;
    }

    private void setAttachmentViewerInfo(IndexerSearchResultItem item) {
        List<String> sources = item.getSchemaElementValue("source", "");
        if (sources != null && !sources.isEmpty()) {
            String source = sources.get(0);
            if (source != null) {
                boolean isZip = source.endsWith(".zip");
                boolean isPage = source.endsWith(".page");
                boolean isZippedWebpage = source.endsWith(".zwp");

                if (isZip || isPage || isZippedWebpage) {
                    item.setArchiveWebpage(true);
                }
            }
        }
        item.setAttachmentViewerUrl(AttachmentViewerController.MAPPING_PREFIX + item.getRecordId());
    }

}
