package hu.kozadat.search.service.viewer;

import hu.kozadat.helper.FilenameConverter;
import hu.kozadat.search.exception.ServiceException;
import hu.kozadat.search.service.SearchUIConfiguration;
import org.apache.commons.io.FileUtils;
import org.fuin.utils4j.Utils4J;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.URL;
import java.net.URLConnection;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.charset.Charset;
import java.util.Collections;
import java.util.Set;
import java.util.TreeSet;

/**
 * Created with IntelliJ IDEA.
 * User: tnagy
 * Date: 2013.04.10.
 * Time: 17:03
 */
@Service
public class AttachmentViewerService implements InitializingBean {

    private static final Logger LOGGER = LoggerFactory.getLogger(AttachmentViewerService.class);

    private static final long HOUR_IN_MILLIS = 60 * 60 * 1000;

    private static final String[] INDEX_FILENAMES = {"index", "fooldal", "main"};

    private static final String MIME_TYPE_APPLICATION_ZIP = "application/zip";
    private static final String EXTENSION_HTML = ".html";
    private static final String EXTENSION_HTM = ".htm";
    private static final String HTTP_HEADER_NAME_CONTENT_TYPE = "Content-Type";
    private static final String HEADERS_INFO_FILE = "headers.info";
    private static final String INDEX_INFO_FILE = "index.info";
    private static final String ENCODING_INFO_FILE = "encoding.info";

    private static final String HTML4_CONTENT_TYPE_DEFINITION = "<meta http-equiv=\"content-type\" content=\"{{ contentType }}\"/>";
    private static final String HTML5_CONTENT_TYPE_DEFINITION = "<meta charset=\"{{ charset }}\"/>";
    private static final String ENCODING_UTF_8 = "UTF-8";
    private static final String STR_EQUAL_SIGN = "=";
    private static final String CONTENT_TYPE_CHARSET_KEY_PART = "charset" + STR_EQUAL_SIGN;

    private SearchUIConfiguration config;
    private Set<String> availableIndexFilenames;

    public boolean isZip(String url) throws ServiceException {
        if (url != null) {
            try {
                return MIME_TYPE_APPLICATION_ZIP.equals(guessMimeType(url));
            } catch (IOException e) {
                LOGGER.error(e.getMessage(), e);
                throw new ServiceException(e.getMessage(), e);
            }
        }
        return false;
    }

    public String downloadAndUnzip(String recordId, String sourceValue) throws ServiceException {

        // tnagy download
        File download = download(recordId, sourceValue);

        // tnagy unzip
        File dir = unzip(download);

        // tnagy clean filenames
        FilenameConverter.cleanAndSafeHtmls(dir);

        // tnagy save info
        saveInfoFiles(dir);

        return getIndexInfo(recordId);
    }

    private void saveInfoFiles(File directory) throws ServiceException {
        File indexHtmlFile = findIndexBySize(directory);

        if (indexHtmlFile == null) {
            return;
        }

        try {

            saveIndexInfo(indexHtmlFile);

            saveEncodingInfo(indexHtmlFile);

        } catch (IOException e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    private void saveIndexInfo(File indexHtmlFile) throws IOException {
        writeIndexInfo(indexHtmlFile.getParentFile(), indexHtmlFile.getName(), /* overwrite */ false);
    }

    private void saveEncodingInfo(File indexHtmlFile) throws IOException, ServiceException {
        Charset charset = guessCharset(indexHtmlFile);
        writeEncodingInfo(indexHtmlFile.getParentFile(), charset, /* overwrite */ false);
    }

    private Charset guessCharset(File indexHtmlFile) throws IOException, ServiceException {
        Charset charset;
        // tnagy find charset by headers
        String contentType = findContentTypeInHeaders(new File(indexHtmlFile.getParentFile().getCanonicalPath() + "/" + HEADERS_INFO_FILE));
        if (contentType != null && !"".equals(contentType.trim()) && contentType.toLowerCase().contains(CONTENT_TYPE_CHARSET_KEY_PART)) {
            charset = getCharset(contentType);
        } else {
            // tnagy find charset by html meta
            charset = getCharset(indexHtmlFile);
        }
        return charset;
    }

    private void writeEncodingInfo(File directory, Charset charset, boolean overwrite) throws IOException {
        // tnagy save charset into encoding.info
        if (charset == null) {
            return;
        }
        File encodingInfoFile = new File(directory.getCanonicalPath() + "/" + ENCODING_INFO_FILE);
        if (overwrite || !encodingInfoFile.exists()) {
            FileUtils.writeStringToFile(encodingInfoFile, charset.name(), ENCODING_UTF_8);
        } else if (encodingInfoFile.exists()) {
            LOGGER.warn("writeEncodingInfo(): encoding.info already exists, detected charset={}, WON'T overwrite.", charset.name());
        }
    }

    private void writeIndexInfo(File directory, String indexHtml, boolean overwrite) throws IOException {
        // tnagy save indexHtml into index.info
        File indexInfoFile = new File(directory.getCanonicalPath() + "/" + INDEX_INFO_FILE);
        if (overwrite || !indexInfoFile.exists()) {
            FileUtils.writeStringToFile(indexInfoFile, indexHtml, ENCODING_UTF_8);
        } else if (indexInfoFile.exists()) {
            LOGGER.warn("writeIndexInfo(): index.info already exists, selected index={}, WON'T overwrite.", indexHtml);
        }
    }

    public boolean hasContent(String recordId) throws ServiceException {
        File destFile = getDestFile(recordId);
        if (destFile == null) {
            return false;
        }
        File parentFile = destFile.getParentFile();
        return parentFile != null && parentFile.exists();
    }

    public String getIndexInfo(String recordId) throws ServiceException {
        File destFile = getDestFile(recordId);
        if (destFile == null) {
            throw new ServiceException("No content root for " + recordId);
        }
        try {
            File indexInfo = new File(destFile.getParentFile().getCanonicalPath() + "/" + INDEX_INFO_FILE);
            if (!indexInfo.exists()) {
                return null;
            }
            return FileUtils.readFileToString(indexInfo).replaceAll("\n", "");
        } catch (IOException e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public String getEncodingInfo(String recordId) throws ServiceException {
        File destFile = getDestFile(recordId);
        if (destFile == null) {
            throw new ServiceException("No content root for " + recordId);
        }
        try {
            File encodingInfo = new File(destFile.getParentFile().getCanonicalPath() + "/" + ENCODING_INFO_FILE);
            if (!encodingInfo.exists()) {
                return null;
            }
            String encodingStr = FileUtils.readFileToString(encodingInfo).replaceAll("\n", "");

            if (encodingStr != null && "".equals(encodingStr.trim())) {
                return null;
            }

            return encodingStr;
        } catch (IOException e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    public String getContentTypeInfo(String recordId) throws ServiceException {
        try {
            File destFile = getDestFile(recordId);
            if (destFile == null) {
                throw new ServiceException("No content root for " + recordId);
            }
            File headersInfo = new File(destFile.getParentFile().getCanonicalPath() + "/" + HEADERS_INFO_FILE);
            return findContentTypeInHeaders(headersInfo);
        } catch (IOException e) {
            throw new ServiceException(e.getMessage(), e);
        }
    }

    @Scheduled(fixedDelay = 300000)
    public void deleteOldAttachmentContents() {
        LOGGER.info("deleteOldAttachmentContents(): CALLED");
        // tnagy check content in every 5 mins

        int deletedContent = 0;

        try {
            String attachmentContentRoot = getAttachmentContentRoot();
            File contentRoot = new File(attachmentContentRoot);

            if (contentRoot.exists() && contentRoot.isDirectory()) {
                long now = System.currentTimeMillis();
                File[] files = contentRoot.listFiles();
                if (files != null) {
                    for (File content : files) {
                        if ((now - content.lastModified()) > HOUR_IN_MILLIS) {
                            if (content.isDirectory()) {
                                FileUtils.deleteDirectory(content);
                            } else {
                                FileUtils.forceDelete(content);
                            }
                            deletedContent += 1;
                        }
                    }
                }
            } else {
                LOGGER.warn("deleteOldAttachmentContents(): {} not exists or not directory!", contentRoot);
            }

        } catch (Exception e) {
            LOGGER.error(e.getMessage(), e);
        }
        LOGGER.info("deleteOldAttachmentContents(): FINISHED, {} files/directories deleted.", deletedContent);
    }

    private Charset getCharset(String contentType) {
        if (contentType == null) {
            return null;
        }
        int beginIndex = contentType.lastIndexOf(STR_EQUAL_SIGN) + 1;
        if (beginIndex == -1) {
            return null;
        }
        if (contentType.lastIndexOf(STR_EQUAL_SIGN) + 1 >= contentType.length()) {
            return null;
        }
        String contentTypePart = contentType.substring(beginIndex);
        return getCharsetByName(contentTypePart);
    }

    private Charset getCharsetByName(String charsetName) {
        if (charsetName == null) {
            return null;
        }
        Charset charset = null;
        try {
            charset = Charset.forName(charsetName);
        } catch (Exception e) {
            LOGGER.error("getCharsetByName(): ERROR: " + charsetName + " - " + e.getMessage(), e);
        }
        return charset;
    }

    private Charset getCharset(File file) throws ServiceException {
        String encoding = null;
        try {
            org.jsoup.nodes.Document document = Jsoup.parse(file, ENCODING_UTF_8);

            Elements metaCharset = document.select("meta[charset]");
            if (!metaCharset.isEmpty()) {
                for (int i = 0; i < metaCharset.size(); i++) {
                    if (encoding == null) {
                        Element element = metaCharset.get(i);
                        String charset = element.attr("charset");
                        if (charset != null && !"".equals(charset)) {
                            encoding = charset;
                        }
                    }
                }
            }

            if (encoding == null) {
                Elements metaContentType = document.select("meta[http-equiv]");
                if (!metaContentType.isEmpty()) {
                    for (int i = 0; i < metaContentType.size(); i++) {
                        if (encoding == null) {
                            Element element = metaContentType.get(i);
                            String httpEquiv = element.attr("http-equiv");
                            if (HTTP_HEADER_NAME_CONTENT_TYPE.equalsIgnoreCase(httpEquiv)) {
                                String contentType = element.attr("content");
                                if (contentType != null && !"".equals(contentType)) {
                                    encoding = contentType.substring(contentType.lastIndexOf("=") + 1);
                                }
                            }
                        }
                    }
                }
            }

        } catch (IOException e) {
            // tnagy swallow exceptions
            LOGGER.error(e.getMessage());
        }

        return getCharsetByName(encoding);

    }

    private boolean isHtmlFile(File file) {
        return file.getName().endsWith(EXTENSION_HTML) || file.getName().endsWith(EXTENSION_HTM);
    }

    private String readFileContentAsString(File file) throws IOException {
        if (file == null || !file.exists()) {
            return null;
        }
        return FileUtils.readFileToString(file, ENCODING_UTF_8);
    }

    private String findContentTypeInHeaders(File file) throws IOException {
        if (file == null || !file.exists()) {
            return null;
        }

        String contentType = null;

        String line;
        BufferedReader fileReader = new BufferedReader(new FileReader(file));
        while ((line = fileReader.readLine()) != null) {
            if (line.startsWith(HTTP_HEADER_NAME_CONTENT_TYPE)) {
                contentType = line.replaceFirst(HTTP_HEADER_NAME_CONTENT_TYPE + ": ", "");
            }
        }
        return contentType;
    }

    private String guessMimeType(String url) throws IOException {
        String mimeType = URLConnection.guessContentTypeFromName(url);
        if (mimeType == null || "content/unknown".equals(mimeType)) {
            URL urlToGuess = new URL(url);
            URLConnection urlConnection = urlToGuess.openConnection();
            urlConnection.setAllowUserInteraction(false);
            String mimeTypeFromUrl = urlConnection.getContentType();
            if (mimeTypeFromUrl != null) {
                mimeType = mimeTypeFromUrl;
            }
        }
        return mimeType;
    }

    private File getDestFile(String recordOaiId) throws ServiceException {

        if (recordOaiId == null) {
            throw new ServiceException("Parameter 'recordOaiId' cannot be null");
        }

        String attachmentContentRoot = getAttachmentContentRoot();

        // tnagy replace all non-word character
        String filename = recordOaiId.replaceAll("\\W", "").concat(".zip");

        return new File(attachmentContentRoot + recordOaiId + "/" + filename);
    }

    private String getAttachmentContentRoot() throws ServiceException {
        String attachmentContentRoot = config.getAttachmentContentRoot();
        if (attachmentContentRoot == null || "".equals(attachmentContentRoot.trim())) {
            throw new ServiceException("attachmentContentRoot is null or empty");
        }
        if (!attachmentContentRoot.endsWith("/")) {
            attachmentContentRoot = attachmentContentRoot + "/";
        }
        return attachmentContentRoot;
    }

    private File findIndexByName(File dir) throws ServiceException {
        if (dir == null || !dir.isDirectory()) {
            throw new ServiceException("unzip() returned invalid dir");
        }

        File index = null;
        File anyHtml = null;

        File[] files = dir.listFiles();
        if (files != null) {
            for (File file : files) {
                if (index == null) {
                    if (availableIndexFilenames.contains(file.getName())) {
                        index = file;
                    }
                }
                if (anyHtml == null) {
                    if (isHtmlFile(file)) {
                        anyHtml = file;
                    }
                }
            }
        }

        if (index == null) {
            index = anyHtml;
        }

        if (index == null) {
            throw new ServiceException("HMTL file not found in dir " + dir);
        }
        return index;
    }

    private File findIndexBySize(File directory) throws ServiceException {
        if (directory == null || !directory.isDirectory()) {
            throw new ServiceException("unzip() returned invalid directory");
        }

        File index = null;
        long maxSize = 0L;

        File[] files = directory.listFiles();
        if (files != null) {
            for (File file : files) {
                if (isHtmlFile(file)) {
                    if (file.length() > maxSize) {
                        index = file;
                        maxSize = index.length();
                    }
                }
            }
        }

        if (index == null) {
            throw new ServiceException("HMTL file not found in dir " + directory);
        }

        return index;
    }

    private File download(String recordId, String sourceValue) throws ServiceException {
        if (recordId == null || "".equals(recordId.trim())) {
            throw new ServiceException("Parameter 'recordId' cannot be null or empty");
        }
        if (sourceValue == null || "".equals(sourceValue.trim())) {
            throw new ServiceException("Parameter 'sourceValue' cannot be null or empty");
        }

        try {
            File destFile = getDestFile(recordId);

            destFile.getParentFile().mkdirs();

            URL url = new URL(sourceValue);
            ReadableByteChannel readableByteChannel = Channels.newChannel(url.openStream());
            FileOutputStream fileOutputStream = new FileOutputStream(destFile);
            fileOutputStream.getChannel().transferFrom(readableByteChannel, 0, Long.MAX_VALUE);

            return destFile;

        } catch (IOException e) {
            LOGGER.error(e.getMessage(), e);
            throw new ServiceException(e.getMessage(), e);
        }

    }

    private File unzip(File zipFile) throws ServiceException {
        try {
            File parentFile = zipFile.getParentFile();
            Utils4J.unzip(zipFile, parentFile);
            return parentFile;
        } catch (IOException e) {
            LOGGER.error(e.getMessage(), e);
            throw new ServiceException(e.getMessage(), e);
        }

    }

    @Autowired
    public void setConfig(SearchUIConfiguration config) {
        this.config = config;
    }

    @Override
    public void afterPropertiesSet() throws Exception {

        Set<String> filenames = new TreeSet<>();

        for (String indexFilename : INDEX_FILENAMES) {
            filenames.add(indexFilename + EXTENSION_HTML);
            filenames.add(indexFilename + EXTENSION_HTM);
        }

        availableIndexFilenames = Collections.unmodifiableSet(filenames);

    }

}
